@@ -1,3 +1,5 @@
+#define QT_FEATURE_version_tagging 1
+
 #define QT_FEATURE_shared 1
 
 #define QT_FEATURE_static -1
@@ -51,17 +53,16 @@
 #define QT_FEATURE_opensslv30 -1
 
 
-#define QT_VERSION_STR "6.6.3"
+#define QT_VERSION_STR "6.8.2"
 
 #define QT_VERSION_MAJOR 6
 
-#define QT_VERSION_MINOR 6
+#define QT_VERSION_MINOR 8
 
-#define QT_VERSION_PATCH 3
+#define QT_VERSION_PATCH 2
 
 #define QT_COPYRIGHT "Copyright (C) The Qt Company Ltd. and other contributors."
 #define QT_LARGEFILE_SUPPORT 64
 #define QT_VISIBILITY_AVAILABLE
 #define QT_USE_PROTECTED_VISIBILITY
-#define QT_COMPILER_SUPPORTS_NEON 1
 #define QT_LINKED_OPENSSL
