@@ -20,6 +20,8 @@
 
 #define QT_FEATURE_vulkan 1
 
+#define QT_FEATURE_metal -1
+
 #define QT_FEATURE_openvg -1
 
 #define QT_FEATURE_egl 1
@@ -106,3 +108,5 @@
 
 #define QT_FEATURE_undogroup 1
 
+#define QT_FEATURE_wayland 1
+
