@@ -12,8 +12,6 @@
 
 #define QT_FEATURE_ocsp 1
 
-#define QT_FEATURE_opensslv11 1
-
 #define QT_FEATURE_sctp -1
 
 #define QT_FEATURE_http 1
@@ -40,7 +38,6 @@
 
 #define QT_FEATURE_topleveldomain 1
 
-#define QT_LINKED_OPENSSL
 #define QT_NO_SCTP 1
 #define QT_NO_GSSAPI 1
 #define QT_NO_SSPI 1
